// parameter list

#define OUT 	8
#define IN 	12
#define COUNT 	16
#define STATE 	20

// void decimbody( fract16 out[], fract16 in[], countBy2R, fir_state_fr16 * s );

// offset definition of fir_state_fr16
#define H	0	/* filter coefficients             */ 
#define D	4	/* start of delay line             */ 
#define P	8	/* read/write pointer              */ 
#define K	12	/* taps + R         */ 
#define L	16	/* interpolation/decimation index  */ 

.section program;
.global _decimbody;
_decimbody:
    link    0;
    [--sp] = (r7:4, p5:3);          	// save registers
    
    p1 = [FP+STATE];		// get state pointer   
    p3 = r0;				// out
    p4 = r1;				// in
    p5 = r2;				// countBy2R
    r0 = [p1++];			// filter coeff
    i0 = r0;
    b0 = i0;
    r0 = [p1++];			// start of delay line
    b1 = r0;
    r0 = [p1++];			// read write pointer
    i1 = r0;
    r0 = [p1++];			// TAPS+R
    r1 = [p1++];			// decimation index (R)
    p2 = r1;
    p2 = p2 << 1;
    p0 = r0;				// TAPS+R
    r0 = r0 << 1;			// length by 16bit word
    l1 = r0;				// 2(TAPS+R) : delay line
    r0 = r0 << 1;			// length by 32bit word
    l0 = r0;				// 4(TAPS+R) : coeff
    
    p0 += -1;
    p2 += -1;
    
    // Here, all variable is ready
    // 	P0 : TAPS+R-1
    //  P2 : 2R-1
    //	P3 : out
    //	P4 : in
    //	P5 : countBy2R
    //	i0, b0, l0 : coeff
    //	i1, b1, l1 : delay line
    
    i0 -= 4;				// coeff points oldest one
    
    loop outerLoop lc0 = p5;			// loop countBy2R times
	loop_begin outerLoop;
						// copy 2R samples iinto delay line
   	    r0 = w[p4++](z);				// get a samples from in buffer
		loop copy2RSamples lc1 = p2;	// loop 2R-1 times;
	    loop_begin copy2RSamples;
	   	    r0 = w[p4++](z)||    w[i1++] = r0.L;
	    loop_end copy2RSamples;
	    w[i1++] = r0.L;					// store 2 samples to delay line
						// now delay  line pointer is set to oldest one
		
						// core FIR filter
		a1 = a0 = 0;					// clear accmulator
	    r0 = [i0--] || r1.L = w[i1++];
	    loop doFIR lc1 = p0;			// loop TAPS+R-1 times
	    loop_begin doFIR;
		    a1 += r0.H * r1.L, a0 += r0.L * r1.L || r0 = [i0--] || r1.L = w[i1++];
	    loop_end doFIR;
	    r2.H = (a1 += r0.H * r1.L), r2.L = (a0 += r0.L * r1.L);
		[p3++] = r2;					// store output
	
	loop_end outerLoop;
    
    p0 = [FP+STATE];			// get state pointer
    l1 = 0;						// recover L1
    l0 = l1;					// recover L0
    r0 = i1;
    p0 += P;					// address of read write pointer
    [p0] = r0;					// store r/w pointer
    (r7:4, p5:3) = [sp++];    	// restore register 
    unlink;
    rts;
_decimbody.end:

/*
    i0 -= 4;				// coeff points oldest one
    
    loop outerLoop lc0 = p5;		// loop countBy2R times
	loop_begin outerLoop;
						// copy 2R samples iinto delay line
		loop copy2RSamples lc1 = p2;	// loop R times;
	    loop_begin copy2RSamples;
	   	    r0 = [p4++];		// get 2 samples from in buffer
		    [i1++] = r0;		// store 2 samples to delay line
	    loop_end copy2RSamples;
						// now delay  line pointer is set to oldest one
		
						// core FIR filter
		a1 = a0 = 0;				// clear accmulator
	    loop doFIR lc1 = p0;		// loop TAPS +R times
	    loop_begin doFIR;
		    r0 = [i0--] || r1.L = w[i1++];
		    a1 += r0.H * r1.L, a0 += r0.L * r1.L;
	    loop_end doFIR;
	   	r2.H = a1, r2.L = a0;
		[p3++] = r2;			// store output
	
	loop_end outerLoop;
    
*/
